# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Affichage des niveaux du jeu sélectionné.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import bge

import utils, input_controller, game_controller



# **************************************************
#       CONTRÔLES DU JEU
# **************************************************
controller = None
scene = None



# **************************************************
#       LES ITEMS DU MENU
# **************************************************
menuItemsList = []



# **************************************************
#       FONCTIONS UTILES
# **************************************************
def postDrawTexts():
    """
    affichage des textes.
    """
    utils.postDrawTexts('levels_loop')

def doAction(what):
    """
    lance l'action correspondant au menu qui a été cliqué.
    """
    for menuItemName in bge.logic.globalDict['TEMP']['menuItems']:
        menuItem = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
        if menuItem['what'] == what:
            break

    if what == 91:
        # retour au menu :
        bge.logic.globalDict['GAME']['state'] = 'levels_quit'
        utils.clearGlobalDict()
        scene.replace('menu')
    else:
        # lancer le jeu :
        gameName = bge.logic.globalDict['MENU']['gameName']
        menu = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
        levelName = menuItem['name']
        bge.logic.globalDict['GAME']['level'] = levelName
        # on remplit le niveau (au cas où on le referait) :
        game_controller.reloadMissions(gameName, levelName)
        bge.logic.globalDict['GAME']['state'] = 'levels_quit'
        utils.clearGlobalDict()
        scene.replace('game')



# **************************************************
#       FONCTION PRINCIPALE
# **************************************************
def main():
    # ---------------------------------------------------------
    # RÉCUPÉRATION DE L'ÉTAT DU JEU
    # ---------------------------------------------------------
    state = bge.logic.globalDict['GAME'].get('state', '')
    if not(state in ('levels_first', 'levels_loop', 'levels_quit')):
        bge.logic.globalDict['GAME']['state'] = 'levels_first'

    # ---------------------------------------------------------
    # PREMIER LANCEMENT
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] == 'levels_first':
        # valeurs initiales (lues depuis le fichier de config si existe) :
        utils.loadSettings()
        lang = bge.logic.globalDict['MENU']['lang']
        global controller, scene, menuItemsList
        controller = bge.logic.getCurrentController()
        scene = bge.logic.getCurrentScene()
        # récupération du nom du jeu :
        gameName = bge.logic.globalDict['MENU']['gameName']
        # affichage des textes :
        scene.post_draw += [postDrawTexts]
        bge.logic.globalDict['TEXTS'] = {}
        utils.setText(
            'progName', text=utils.PROGNAME, 
            color=utils.colors['blue'], width=2, position=(0.07, 0.12))
        text = utils.tr('GAME_{0}'.format(gameName), gameName, lang)
        utils.setText(
            'gameName', text=text,
            color=utils.colors['white'], width=4, position=(0.3, 0.12))
        utils.setText(
            'comment', text='', 
            color=utils.colors['white'], width=2, position=(0.35, 0.3))
        # création des menus :
        bge.logic.globalDict['TEMP']['menuItems'] = {}
        bge.logic.globalDict['TEMP']['menuOver'] = ''
        menuItemsList = []
        # les menus des niveaux :
        what, i = -1, 0
        firstLevel = True
        for [level, ok] in bge.logic.globalDict['GAMES'][gameName]:
            what += 1
            if level['name'] != '':
                i += 1
                menuItemName = 'item_{0}'.format(what)
                position = (0.05, 0.2 + 0.1 * i)
                if (ok == False) and firstLevel:
                    firstLevel = False
                    menuOk = True
                else:
                    menuOk = ok
                utils.createMenuItem(
                    menuItemName, 
                    name=level['name'], 
                    text=utils.tr(
                        'GAME_{0}'.format(gameName), level['name'], lang),
                    comment=utils.tr(
                        'GAME_{0}'.format(gameName), '{0}_COMMENT'.format(level['name']), lang),
                    width=1.5, 
                    position=position, 
                    what=what, 
                    ok=menuOk)
                if menuOk:
                    bge.logic.globalDict['TEMP']['enterAction'] = what
                    menuItemsList.append(what)
        # retour au menu principal :
        utils.createMenuItem(
            'item_91', name='MENU', text=utils.tr('menu', 'MENU', lang),
            position=(0.65, 0.95), what=91)
        menuItemsList.append(91)
        # pour les tests :
        bge.logic.globalDict['TEMP']['TEST_KEY'] = False
        # on affiche la souris :
        bge.logic.mouse.position = bge.logic.globalDict['GAME']['cursorPosition']
        bge.render.showMouse(True)
        # initialisation terminée :
        bge.logic.globalDict['GAME']['state'] = 'levels_loop'

    # ---------------------------------------------------------
    # BOUCLE DES ÉVÉNEMENTS
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] != 'levels_loop':
        return
    # récupération des coordonnées de la souris :
    cursorPosition = input_controller.getCursorPosition()
    # affichage des menus et récupération de l'action active :
    what = -1
    for menuItemName in bge.logic.globalDict['TEMP']['menuItems']:
        menuItem = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
        if menuItem['ok']:
            menuPosition = menuItem['position']
            over = input_controller.mouseOver(cursorPosition, menuPosition)
            if over:
                what = menuItem['what']
                lang = bge.logic.globalDict['MENU']['lang']
                if menuItemName != bge.logic.globalDict['TEMP']['menuOver']:
                    bge.logic.globalDict['TEMP']['menuOver'] = menuItemName
                    menuItem['width'] = menuItem['initialWidth'] * 1.5
                    utils.setText(menuItemName, width=menuItem['width'])
                    utils.playSound('over')
                    utils.setText('comment', text=menuItem['comment'])
            else:
                menuItem['width'] = menuItem['initialWidth']
                utils.setText(menuItemName, width=menuItem['width'])
                if menuItemName == bge.logic.globalDict['TEMP']['menuOver']:
                    bge.logic.globalDict['TEMP']['menuOver'] = ''
    if what < 0:
        utils.setText('comment', text='')
    inputState = input_controller.inputManagement('menu')
    # pour faire un screencast :
    if inputState['SCREENSHOT']:
        utils.makeScreenshot()
    # gestion des menus et autres actions :
    if (what > -1) and (inputState['ENTER']):
        utils.playSound('clic')
        doAction(what)
        return
    elif inputState['QUIT']:
        doAction(91)
        return
    elif ('enterAction' in bge.logic.globalDict['TEMP']):
        if inputState['ENTER']:
            doAction(bge.logic.globalDict['TEMP']['enterAction'])
            return
    # touche test pour débloquer les niveaux :
    testKey = bge.logic.globalDict['TEMP'].get('TEST_KEY', False)
    if not(testKey):
        if inputState['TEST']:
            bge.logic.globalDict['TEMP']['TEST_KEY'] = True
            for menuItemName in bge.logic.globalDict['TEMP']['menuItems']:
                menuItem = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
                menuItem['ok'] = True
                utils.setText(menuItemName, color=utils.colors['orange'])
    if inputState['NEXT']:
        what = utils.nextInList(menuItemsList, what)
        menuItem = 'item_{0}'.format(what)
        menuPosition = bge.logic.globalDict['TEMP']['menuItems'][menuItem]['position']
        bge.logic.mouse.position = (menuPosition[0] + 0.02, menuPosition[1])


