# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Gestion des entrées par le joueur.
    * souris
    * clavier
    * manette
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import bge



# **************************************************
#       CONSTANTES D'ÉTAT
# **************************************************
JUST_ACTIVATED = bge.logic.KX_INPUT_JUST_ACTIVATED
ACTIVE = bge.logic.KX_INPUT_ACTIVE



# **************************************************
#       CONSTANTES CLAVIER
# **************************************************
QUIT_KEY = bge.events.F8KEY#QKEY
SCREENSHOT_KEY = bge.events.F11KEY
HELP_KEY = bge.events.F1KEY
PAUSE_KEY = bge.events.F2KEY
MESSAGE_KEY = bge.events.F3KEY
LOCALHOST_KEY = bge.events.LKEY
CAMERA_KEY = bge.events.CKEY
TEST_KEY = bge.events.F12KEY

UP_ARROW_KEY = bge.events.UPARROWKEY
DOWN_ARROW_KEY = bge.events.DOWNARROWKEY
LEFT_ARROW_KEY = bge.events.LEFTARROWKEY
RIGHT_ARROW_KEY = bge.events.RIGHTARROWKEY

ENTER_KEY = bge.events.ENTERKEY
ESC_KEY = bge.events.ESCKEY
SPACE_KEY = bge.events.SPACEKEY
TAB_KEY = bge.events.TABKEY
BACKSPACE_KEY = bge.events.BACKSPACEKEY
LEFT_SHIFT_KEY = bge.events.LEFTSHIFTKEY
RIGHT_SHIFT_KEY = bge.events.RIGHTSHIFTKEY
CAPSLOCK_KEY = bge.events.CAPSLOCKKEY



# **************************************************
#       CONSTANTES ET FONCTIONS SOURIS
# **************************************************
LEFT_MOUSE = bge.events.LEFTMOUSE
RIGHT_MOUSE = bge.events.RIGHTMOUSE
MIDDLE_MOUSE = bge.events.MIDDLEMOUSE
WHEEL_UP_MOUSE = bge.events.WHEELUPMOUSE
WHEEL_DOWN_MOUSE = bge.events.WHEELDOWNMOUSE

def mouseOver(mousePosition, menuPosition):
    """
    détecte si la souris est au-dessus d'un menu.
    """
    if (mousePosition[0] > menuPosition[0] - 0.02) \
        and (mousePosition[0] < menuPosition[0] + 0.2):
        if (mousePosition[1] > menuPosition[1] - 0.04) \
            and (mousePosition[1] < menuPosition[1] + 0.04):
            return True
    return False

def getCursorPosition():
    """
    retourne la position du curseur
    et met à jour la variable globale (dans globalDict).
    """
    cursorPosition = bge.logic.mouse.position
    bge.logic.globalDict['GAME']['cursorPosition'] = cursorPosition
    return cursorPosition



# **************************************************
#       ÉTAT DU JOYSTICK
# **************************************************
JOYSTICK_STATE = {
    'detected': False, 
    'joystick': None, 
    'activeButtons': [],
    'newActivatedButtons': [],
    'axisValues': [0, 0],
    }

def getJoystickState():
    global JOYSTICK_STATE
    if not(JOYSTICK_STATE['detected']):
        JOYSTICK_STATE['detected'] = True
        JOYSTICK_STATE['joystick'] = bge.logic.joysticks[0]
    JOYSTICK_STATE['newActivatedButtons'] = []
    joystick = JOYSTICK_STATE['joystick']
    if joystick != None:
        if joystick.activeButtons != JOYSTICK_STATE['activeButtons']:
            for button in joystick.activeButtons:
                if not(button in JOYSTICK_STATE['activeButtons']):
                    JOYSTICK_STATE['newActivatedButtons'].append(button)
            JOYSTICK_STATE['activeButtons'] = joystick.activeButtons
        JOYSTICK_STATE['axisValues'] = joystick.axisValues[:2]



# **************************************************
#       
# **************************************************
def inputManagement(what='game'):
    """
    gestion des évènements d'entrée (clavier, souris, manette).
    Retourne un dico (inputState) que le jeu peut traiter sans se 
    préoccuper du périphérique d'entrée utilisé.
    what : (splash, menu, game) indique dans quel type de scène on est.
    """
    inputState = {}
    keyboard = bge.logic.keyboard.events
    mouse = bge.logic.mouse.events
    getJoystickState()

    if what == 'splash':
        # seules les photos d'écran sont gérées
        inputState = {'SCREENSHOT': False,}
        # évènements internes :
        if bge.logic.globalDict['MENU']['video']:
            # enregistrement d'une vidéo en cours :
            inputState['SCREENSHOT'] = True
        # évènements clavier :
        if keyboard[SCREENSHOT_KEY] == JUST_ACTIVATED:
            # on fait une photo d'écran :
            inputState['SCREENSHOT'] = True
    elif what == 'menu':
        # scène de type menu (ou réglages réseau)
        inputState = {
            'SCREENSHOT': False,
            'HELP': False,
            'QUIT': False,
            'LOCALHOST': False,
            'TEST': False,
            'ENTER': False,
            'NEXT': False,
            }
        # évènements internes :
        if bge.logic.globalDict['MENU']['video']:
            # enregistrement d'une vidéo en cours :
            inputState['SCREENSHOT'] = True
        # évènements clavier :
        if keyboard[SCREENSHOT_KEY] == JUST_ACTIVATED:
            # on fait une photo d'écran :
            inputState['SCREENSHOT'] = True
        elif keyboard[HELP_KEY] == JUST_ACTIVATED:
            # aide en ligne :
            inputState['HELP'] = True
        elif keyboard[ESC_KEY] == JUST_ACTIVATED:
            # on quitte :
            inputState['QUIT'] = True
        elif keyboard[LOCALHOST_KEY] == JUST_ACTIVATED:
            # on passe en localhost :
            inputState['LOCALHOST'] = True
        elif keyboard[TEST_KEY] == JUST_ACTIVATED:
            # pour faire des tests :
            inputState['TEST'] = True
        elif keyboard[TAB_KEY] == JUST_ACTIVATED:
            # on passe au menu suivant :
            inputState['NEXT'] = True
        elif keyboard[ENTER_KEY] == JUST_ACTIVATED:
            # on valide :
            inputState['ENTER'] = True
        # évènements souris :
        if mouse[LEFT_MOUSE] == JUST_ACTIVATED:
            # clic pour valider :
            inputState['ENTER'] = True
        # évènements joystick :
        if JOYSTICK_STATE['joystick'] != None:
            for i in range(4):
                if 2 * i in JOYSTICK_STATE['newActivatedButtons']:
                    # on valide :
                    inputState['ENTER'] = True
                elif 2 * i + 1 in JOYSTICK_STATE['newActivatedButtons']:
                    # on quitte :
                    inputState['QUIT'] = True
            # déplacement de la souris avec la manette :
            velocity = 0.01
            position = list(bge.logic.mouse.position)
            if JOYSTICK_STATE['axisValues'][1] < 0:
                position[1] -= velocity
            elif JOYSTICK_STATE['axisValues'][1] > 0:
                position[1] += velocity
            if JOYSTICK_STATE['axisValues'][0] < 0:
                position[0] -= velocity
            elif JOYSTICK_STATE['axisValues'][0] > 0:
                position[0] += velocity
            bge.logic.mouse.position = tuple(position)
    elif what == 'game':
        # dans le jeu on gère aussi les déplacements
        inputState = {
            'SCREENSHOT': False,
            'PAUSE': False,
            'QUIT': False,
            'MESSAGE': False,
            'TEST': False,
            'ENTER': False,
            'PLAYER_UP': False,
            'PLAYER_DOWN': False,
            'PLAYER_LEFT': False,
            'PLAYER_RIGHT': False,
            'PLAYER_JUMP': False,
            'CAMERA': False,
            }
        # évènements internes :
        if bge.logic.globalDict['MENU']['video']:
            # enregistrement d'une vidéo en cours :
            inputState['SCREENSHOT'] = True
        # évènements clavier :
        if keyboard[SCREENSHOT_KEY] == JUST_ACTIVATED:
            # on fait une photo d'écran :
            inputState['SCREENSHOT'] = True
        elif keyboard[PAUSE_KEY] == JUST_ACTIVATED:
            # on met le jeu en pause :
            inputState['PAUSE'] = True
        elif keyboard[ESC_KEY] == JUST_ACTIVATED:
            # on quitte :
            inputState['QUIT'] = True
        elif keyboard[MESSAGE_KEY] == JUST_ACTIVATED:
            # on réaffiche le message de la mission :
            inputState['MESSAGE'] = True
        elif keyboard[TEST_KEY] == JUST_ACTIVATED:
            # pour faire des tests :
            inputState['TEST'] = True
        elif keyboard[ENTER_KEY] == JUST_ACTIVATED:
            # on valide :
            inputState['ENTER'] = True
        # déplacements suivant le périphérique choisi :
        inputMode = bge.logic.globalDict['MENU']['mode']
        if inputMode == 'keyboard':
            if keyboard[UP_ARROW_KEY] == ACTIVE:
                inputState['PLAYER_UP'] = True
            elif keyboard[DOWN_ARROW_KEY] == ACTIVE:
                inputState['PLAYER_DOWN'] = True
            if keyboard[LEFT_ARROW_KEY] == ACTIVE:
                inputState['PLAYER_LEFT'] = True
            elif keyboard[RIGHT_ARROW_KEY] == ACTIVE:
                inputState['PLAYER_RIGHT'] = True
            if keyboard[SPACE_KEY] == JUST_ACTIVATED:
                inputState['PLAYER_JUMP'] = True
            if keyboard[CAMERA_KEY] == JUST_ACTIVATED:
                inputState['CAMERA'] = True
        elif inputMode == 'joystick':
            if JOYSTICK_STATE['axisValues'][1] < 0:
                inputState['PLAYER_UP'] = True
            elif JOYSTICK_STATE['axisValues'][1] > 0:
                inputState['PLAYER_DOWN'] = True
            if JOYSTICK_STATE['axisValues'][0] < 0:
                inputState['PLAYER_LEFT'] = True
            elif JOYSTICK_STATE['axisValues'][0] > 0:
                inputState['PLAYER_RIGHT'] = True
            for i in range(4):
                if 2 * i in JOYSTICK_STATE['newActivatedButtons']:
                    inputState['PLAYER_JUMP'] = True
                elif 2 * i + 1 in JOYSTICK_STATE['newActivatedButtons']:
                    inputState['CAMERA'] = True
        elif inputMode == 'mouse':
            if mouse[LEFT_MOUSE]:
                inputState['PLAYER_UP'] = True
                # lorsqu'on avance c'est plus facile d'utiliser
                # le clic droit pour les sauts :
                if mouse[RIGHT_MOUSE] == JUST_ACTIVATED:
                    inputState['PLAYER_JUMP'] = True
            elif mouse[RIGHT_MOUSE]:
                inputState['PLAYER_DOWN'] = True
            if mouse[MIDDLE_MOUSE] == JUST_ACTIVATED:
                inputState['PLAYER_JUMP'] = True
            if mouse[WHEEL_UP_MOUSE] == JUST_ACTIVATED:
                inputState['CAMERA'] = True
            elif mouse[WHEEL_DOWN_MOUSE] == JUST_ACTIVATED:
                inputState['CAMERA'] = True
    return inputState

def calcTurn(inputState, player, fps):
    """
    calcul de l'angle de rotation du personnage.
    Dépend du périphérique d'entrée.
    """
    turn = 0
    inputMode = bge.logic.globalDict['MENU']['mode']
    if inputMode == 'keyboard':
        speedTurn = 2
        if inputState['PLAYER_LEFT']:
            turn = speedTurn / fps
        elif inputState['PLAYER_RIGHT']:
            turn = - speedTurn / fps
    elif inputMode == 'mouse':
        # center mouse on first frame, create temp variables
        if not('mouse' in player):
            bge.logic.mouse.position = (0.5, 0.5)
            player['mouse'] = (0.0, 0.0)
        x = round(0.5 - bge.logic.mouse.position[0], 2)
        y = round(0.5 - bge.logic.mouse.position[1], 2)
        # mouse sensitivity
        sensitivity = 1.0
        x *= sensitivity
        y *= sensitivity
        # Smooth movement
        (oldX, oldY) = player['mouse']
        x = (oldX * 0.9 + x * 0.1)
        y = (oldY * 0.9 + y * 0.1)
        player['mouse'] = (x, y)
        # set the values
        turn = x
        # Center mouse in game window
        bge.logic.mouse.position = (0.5, 0.5)
    elif inputMode == 'joystick':
        speedTurn = 2
        if inputState['PLAYER_LEFT']:
            turn = speedTurn / fps
        elif inputState['PLAYER_RIGHT']:
            turn = - speedTurn / fps
    return turn


